/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.facade.runtime.Label;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.light.LightProcessInstance;

/**
 * Internal use only.
 */
public interface WebAPI {

  List<Label> getSystemLabels(String ownerName);

  List<Label> getUserCustomLabels(String ownerName);

  Label getLabel(String ownerName, String labelName);

  Set<Label> getLabels(String ownerName);
  
  void addLabel(String labelName, String ownerName, String editableCSSStyleName, String readonlyCSSStyleName, 
      String previewCSSStyleName, boolean isVisible, boolean hasToBeDisplayed, String iconCSSStyle, 
      Set<ProcessInstanceUUID> caseList, int displayOrder, boolean isSystemLabel);

  void removeLabel(String ownerName, String labelName);

  void removeLabels(String ownerName, Collection<String> labelNames);

  void addCasesToLabel(String ownerName, String labelName, Set<ProcessInstanceUUID> caseList);

  void removeCasesFromLabel(String ownerName, String labelName, Set<ProcessInstanceUUID> caseList);

  void updateLabelCSS(String ownerName, String labelName, String aEditableCSSStyle, String aPreviewCSSStyle, String aReadOnlyCSSStyle);

  void updateLabelVisibility(String ownerName, String labelName, boolean isVisible);

  void updateLabelVisibility(String ownerName, Map<String, Boolean> labelvisibilities);

  void updateLabelName(String ownerName, String labelName, String newName);
  
  Set<Label> getCaseLabels(String ownerName, ProcessInstanceUUID case_);

  Map<ProcessInstanceUUID, Set<Label>> getCasesLabels(String ownerName, Set<ProcessInstanceUUID> cases);

  void removeAllCasesFromLabels(Set<ProcessInstanceUUID> caseList);

  boolean setLabelsOnNewOrUpdatedCases(String ownerName, Set<String> labelsToApplyOnNewOrUpdated, Set<String> labelsToRemoveOnNewOrUpdated, String labelToApplyOnMyCases);

  Set<Label> getLabels(String ownerName, Set<String> labelsName);
  
  Map<String, Integer> getCasesNumber(String owner, Collection<String> labelNames, int limit);
  Map<String, Integer> getCasesNumber(String owner, Collection<String> labelNames);
  Map<String, Integer> getCasesNumber(String owner, String labelName, Collection<String> labelNames, int limit);
  
  Set<ProcessInstanceUUID> getCases(String ownerName, Set<String> labels);
  
  List<LightProcessInstance> getLightProcessInstances(String ownerName, Set<String> theLabelsName, int startingIndex, int maxElementCount);

  void deletePhantomCases();
  void deleteAllCases();
  
  String generateTemporaryToken(String identityKey);
  String getIdentityKeyFromTemporaryToken(String token);
}
