/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade.def.majorElement;

import java.io.Serializable;

import org.ow2.bonita.facade.uuid.TransitionDefinitionUUID;

/**
 * Activities are related to one another via flow control conditions (transition information).
 * Each individual transition has three elementary properties, the from-activity, the to-activity and the condition
 * under which the transition is made.
 * Transition from one activity to another may be conditional (involving expressions which are evaluated
 * to permit or inhibit the transition) or unconditional.
 * This interface is derived from the Transition Information of XPDL.
 */
public interface TransitionDefinition extends ProcessElement, Serializable {

  /**
   * Returns the UUID for the TransitionDefinition.
   */
  TransitionDefinitionUUID getUUID();

  /**
   * Returns the activity from which the transition goes out.
   */
  String getFrom();

  /**
   * Returns the activity to which the transition goes in.
   */
  String getTo();

  /**
   * Returns the condition.
   */
  String getCondition();

  /**
   * Returns true if this transition is a default transition
   */
  boolean isDefault();
}
