/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.ejb.ejb2;

import java.lang.reflect.Proxy;
import java.rmi.RemoteException;
import java.util.Hashtable;

import javax.ejb.CreateException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import org.ow2.bonita.facade.BAMAPI;
import org.ow2.bonita.facade.QueryAPIAccessor;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.ejb.ejb2.home.EJB2BAMAPIHome;
import org.ow2.bonita.facade.ejb.ejb2.home.EJB2QueryDefinitionAPIHome;
import org.ow2.bonita.facade.ejb.ejb2.home.EJB2QueryRuntimeAPIHome;
import org.ow2.bonita.facade.exception.BonitaInternalException;
import org.ow2.bonita.facade.interceptor.ClientAPIInterceptor;
import org.ow2.bonita.facade.internal.InternalBAMAPI;
import org.ow2.bonita.facade.internal.InternalQueryDefinitionAPI;
import org.ow2.bonita.facade.internal.InternalQueryRuntimeAPI;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.Misc;

public class EJB2QueryAPIAccessorImpl implements QueryAPIAccessor {

  private InternalQueryDefinitionAPI queryDefinitionAPI;
  private InternalQueryRuntimeAPI queryRuntimeAPI;
  private InternalBAMAPI bamAPI;
  protected Hashtable<String, String> jndiEnvironment; 
  
  public EJB2QueryAPIAccessorImpl(Hashtable<String, String> jndiEnvironment) {
    this.jndiEnvironment = jndiEnvironment;
  }
  
  public QueryDefinitionAPI getQueryDefinitionAPI() {
    return getQueryDefinitionAPI(QueryRuntimeAPI.DEFAULT_QUERIER);
  }
  
  public QueryDefinitionAPI getQueryDefinitionAPI(final String queryList) {
    if (this.queryDefinitionAPI == null) {
      try {
        final EJB2QueryDefinitionAPIHome queryDefinitionHome = (EJB2QueryDefinitionAPIHome) PortableRemoteObject.narrow(
            Misc.<Object>lookup(AccessorUtil.QUERYDEFINITION_JNDINAME, jndiEnvironment), EJB2QueryDefinitionAPIHome.class);
        this.queryDefinitionAPI = queryDefinitionHome.create();
      } catch (final RemoteException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (final CreateException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (final NamingException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      }
    }
    final Class<QueryDefinitionAPI> clazz = QueryDefinitionAPI.class;
    final ClientAPIInterceptor apiInterceptor = new ClientAPIInterceptor(this.queryDefinitionAPI, queryList);
    return (QueryDefinitionAPI) Proxy.newProxyInstance(QueryAPIAccessor.class.getClassLoader(),
        new Class[] {clazz}, apiInterceptor);
  }

  public QueryRuntimeAPI getQueryRuntimeAPI() {
    return getQueryRuntimeAPI(QueryRuntimeAPI.DEFAULT_QUERIER);
  }

  public QueryRuntimeAPI getQueryRuntimeAPI(final String queryList) {
    if (this.queryRuntimeAPI == null) {
      try {
        final EJB2QueryRuntimeAPIHome queryRuntimeAPIHome = (EJB2QueryRuntimeAPIHome) PortableRemoteObject.narrow(
            Misc.<Object>lookup(AccessorUtil.QUERYRUNTIME_JNDINAME, jndiEnvironment), EJB2QueryRuntimeAPIHome.class);
        this.queryRuntimeAPI = queryRuntimeAPIHome.create();
      } catch (final RemoteException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (final CreateException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (final NamingException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      }
    }
    final Class<QueryRuntimeAPI> clazz = QueryRuntimeAPI.class;
    final ClientAPIInterceptor apiInterceptor = new ClientAPIInterceptor(this.queryRuntimeAPI, queryList);
    return (QueryRuntimeAPI) Proxy.newProxyInstance(QueryAPIAccessor.class.getClassLoader(),
        new Class[] {clazz}, apiInterceptor);
  }

  public BAMAPI getBAMAPI() {
    return getBAMAPI(BAMAPI.DEFAULT_QUERIER);
  }

  public BAMAPI getBAMAPI(final String queryList) {
    if (this.bamAPI == null) {
      try {
        final EJB2BAMAPIHome bamAPIHome = (EJB2BAMAPIHome) PortableRemoteObject.narrow(
            Misc.<Object>lookup(AccessorUtil.BAMAPI_JNDINAME, jndiEnvironment), EJB2BAMAPIHome.class);
        this.bamAPI = bamAPIHome.create();
      } catch (final RemoteException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (final CreateException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (final NamingException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      }
    }
    final Class<BAMAPI> clazz = BAMAPI.class;
    final ClientAPIInterceptor apiInterceptor = new ClientAPIInterceptor(this.bamAPI, queryList);
    return (BAMAPI) Proxy.newProxyInstance(QueryAPIAccessor.class.getClassLoader(),
        new Class[] {clazz}, apiInterceptor);
  } 
}
