/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.identity.impl;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.ow2.bonita.facade.identity.User;
import org.ow2.bonita.util.Misc;

/**
 * @author Anthony Birembaut
 *
 */
public class UserImpl implements User {

  /**
   * UID
   */
  private static final long serialVersionUID = 5357217264648978573L;

  protected long dbid;
  protected String uuid;
  protected String email;
  protected String firstName;
  protected String lastName;
  protected String password;
  protected String username;
  protected Set<RoleImpl> roles;
  
  protected UserImpl() {}
  
  public UserImpl(String username, String password) {
    Misc.checkArgsNotNull(username, password);
    this.uuid = UUID.randomUUID().toString();
    this.username = username;
    this.password = password;
  }
  
  public UserImpl(UserImpl src) {
    Misc.checkArgsNotNull(src);
    this.uuid = src.getUUID();
    this.email = src.getEmail();
    this.firstName = src.getFirstName();
    this.lastName = src.getLastName();
    this.password = src.getPassword();
    this.username = src.getUsername();
  }
  
  public String getEmail() {
    return email;
  }

  public String getFirstName() {
    return firstName;
  }

  public String getLastName() {
    return lastName;
  }

  public String getPassword() {
    return password;
  }

  public Set<RoleImpl> getRoles() {
    if (roles == null) {
      roles = new HashSet<RoleImpl>();
    }
    return roles;
  }

  public String getUsername() {
    return username;
  }

  public void setEmail(String email) {
    this.email = email;
  }

  public void setFirstName(String firstName) {
    this.firstName = firstName;
  }

  public void setLastName(String lastName) {
    this.lastName = lastName;
  }

  public void setPassword(String password) {
    this.password = password;
  }

  public void setUsername(String username) {
    this.username = username;
  }

  public void setRoles(Set<RoleImpl> roles) {
    this.roles = roles;
  }

  public void addRole(RoleImpl role) {
    if (roles == null) {
      roles = new HashSet<RoleImpl>();
    }
    this.roles.add(role);
    role.getUsers().add(this);
  }
  
  public void removeRole(RoleImpl role) {
    if (roles != null && !roles.isEmpty()) {
      roles.remove(role);
      role.getUsers().remove(this);
    }
  }
  @Deprecated
  public long getId() {
    return dbid;
  }
  public String getUUID() {
    return uuid;
  }
  @Deprecated
  public void setUUID(String uuid) {
    this.uuid = uuid;
  }
}
