/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.internal;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Set;

import org.ow2.bonita.facade.exception.RoleAlreadyExistsException;
import org.ow2.bonita.facade.exception.RoleNotFoundException;
import org.ow2.bonita.facade.exception.UserAlreadyExistsException;
import org.ow2.bonita.facade.exception.UserNotFoundException;
import org.ow2.bonita.facade.identity.Role;
import org.ow2.bonita.facade.identity.User;

/**
 * @author Anthony Birembaut
 *
 */
public interface InternalIdentityAPI extends Remote {

  User getUser(String username) throws RemoteException, UserNotFoundException;
  Role getRole(String name) throws RemoteException, RoleNotFoundException;
  
  Set<User> getUsers() throws RemoteException;
  Set<Role> getRoles() throws RemoteException;
  
  Set<Role> getUserRoles(String username) throws RemoteException, UserNotFoundException;
  Set<User> getUsersInRole(String name) throws RemoteException, RoleNotFoundException;
  
  User addUser(String username, String password) throws RemoteException, UserAlreadyExistsException;
  User addUser(String username, String password, String firstName, String lastName, String email) throws RemoteException, UserAlreadyExistsException;
  User updateUser(String oldUsername, String username, String password, String firstName, String lastName, String email) throws RemoteException, UserNotFoundException, UserAlreadyExistsException;
  
  Role addRole(String name) throws RemoteException, RoleAlreadyExistsException;
  Role addRole(String name, String label, String description) throws RemoteException, RoleAlreadyExistsException;
  Role updateRole(String oldName, String name, String label, String description) throws RemoteException, RoleNotFoundException, RoleAlreadyExistsException;
  void setUserRoles(String username, Set<String> roleNames) throws RemoteException, UserNotFoundException, RoleNotFoundException;
  
  void addRoleToUser(String roleName, String username) throws RemoteException, UserNotFoundException, RoleNotFoundException;
  void removeRoleFromUser(String roleName, String username) throws RemoteException, UserNotFoundException, RoleNotFoundException;
  
  void removeUser(String username) throws RemoteException, UserNotFoundException;
  void removeRole(String name) throws RemoteException, RoleNotFoundException;
}
