/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade.internal;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.ow2.bonita.facade.def.element.BusinessArchive;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.DeploymentException;
import org.ow2.bonita.facade.exception.PrivilegeNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.RuleAlreadyExistsException;
import org.ow2.bonita.facade.exception.RuleNotFoundException;
import org.ow2.bonita.facade.exception.UndeletableInstanceException;
import org.ow2.bonita.facade.exception.UndeletableProcessException;
import org.ow2.bonita.facade.exception.UserNotFoundException;
import org.ow2.bonita.facade.privilege.PrivilegePolicy;
import org.ow2.bonita.facade.privilege.Rule;
import org.ow2.bonita.facade.privilege.Rule.RuleType;
import org.ow2.bonita.facade.uuid.AbstractUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;

/**
 * For internal use only.
 */
public interface InternalManagementAPI extends Remote {

  ProcessDefinition deploy(final BusinessArchive businessARchive) throws DeploymentException, RemoteException;

  void deployJar(String jarName, byte[] jar) throws DeploymentException, RemoteException;

  void removeJar(String jarName) throws DeploymentException, RemoteException;

  void deleteProcess(ProcessDefinitionUUID processUUID) throws ProcessNotFoundException,
  UndeletableProcessException, UndeletableInstanceException, RemoteException;

  void deleteAllProcesses() throws UndeletableProcessException, UndeletableInstanceException, RemoteException;

  String getLoggedUser() throws RemoteException;

  Set<String> getAvailableJars() throws RemoteException;
  
  void addMetaData(String key, String value) throws RemoteException;

  String getMetaData(String key) throws RemoteException;

  void deleteMetaData(String key) throws RemoteException;

  void disable(final ProcessDefinitionUUID processUUID) throws DeploymentException, RemoteException;

  void enable(final ProcessDefinitionUUID processUUID) throws DeploymentException, RemoteException;

  void archive(final ProcessDefinitionUUID processUUID) throws DeploymentException, RemoteException;
  
  void disable(final Collection<ProcessDefinitionUUID> processUUIDs) throws DeploymentException, RemoteException;

  void enable(final Collection<ProcessDefinitionUUID> processUUIDs) throws DeploymentException, RemoteException;

  void archive(final Collection<ProcessDefinitionUUID> processUUIDs) throws DeploymentException, RemoteException;

  void delete(final Collection<ProcessDefinitionUUID> processUUIDs)	throws DeploymentException, ProcessNotFoundException,
      UndeletableProcessException, UndeletableInstanceException, RemoteException;
  
  boolean isUserAdmin(String username) throws UserNotFoundException, RemoteException;
  
  boolean checkUserCredentials(String username, String password) throws RemoteException;
     
  <E extends AbstractUUID> Rule createRule(String name, String label, String decription, Set<E> exceptions, RuleType type) throws  RuleAlreadyExistsException, RemoteException;
  void deleteRule(String name) throws RuleNotFoundException, RemoteException;
  <E extends AbstractUUID>void addExceptionsToRule(String ruleName, Set<E> exceptions) throws RuleNotFoundException, RemoteException ;
  <E extends AbstractUUID> void removeExceptionsFromRule(String ruleName, Set<E> exceptions) throws RuleNotFoundException, RemoteException;
  void updateRule(String name, Rule newRule) throws RuleNotFoundException, RemoteException;
  Rule getRule(String name) throws RuleNotFoundException, RemoteException;
  Set<Rule> getRules() throws RemoteException;
  List<Rule> getRules(RuleType ruleType, int fromIndex, int pageSige) throws RemoteException;
  Set<Rule> getRules(Set<String> ruleNames) throws RemoteException, RuleNotFoundException;
  
  void applyRulesToEntity(String ruleName, Set<String> rules) throws RuleNotFoundException, RemoteException;
  void removeRulesFromEntity(String entityID, Set<String> rules) throws RuleNotFoundException, PrivilegeNotFoundException, RemoteException;
  void applySameRulesToOtherEntities(String entityID, Set<String> entities) throws PrivilegeNotFoundException, RemoteException;
  
  Set<Rule> getApplicableRules(String entityID, RuleType ruleType) throws RemoteException;
  Set<Rule> getAllApplicableRules(String entityID) throws RemoteException;
  
  PrivilegePolicy getDefaultPolicy() throws RemoteException;
  void setDefaultPolicy(PrivilegePolicy newPolicy, boolean negateRules) throws RemoteException;
  
  long getNumberOfRules(RuleType ruleType) throws RemoteException;
  

}
