/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Charles Souillard - BonitaSoft S.A.
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 * Modified by Nicolas Chabanoles - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade.internal;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.AttachmentInstance;
import org.ow2.bonita.facade.runtime.Comment;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.light.LightActivityInstance;
import org.ow2.bonita.light.LightProcessInstance;
import org.ow2.bonita.light.LightTaskInstance;

/**
 * For internal use only.
 *
 * */
public interface InternalQueryRuntimeAPI extends Remote {

  ProcessInstance getProcessInstance(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException;
  LightProcessInstance getLightProcessInstance(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException;
  
  Set<ProcessInstance> getProcessInstances(String queryList) throws RemoteException;
  Set<LightProcessInstance> getLightProcessInstances(String queryList) throws RemoteException;
  List<LightProcessInstance> getLightProcessInstances(int fromIndex, int pageSize, String queryList) throws RemoteException;
  
  Set<ProcessInstance> getProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, String queryList) throws RemoteException;
  Set<LightProcessInstance> getLightProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, String queryList) throws RemoteException;
  List<LightProcessInstance> getLightProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, int fromIndex, int pageSize, String queryList) throws RemoteException;
  
  Set<LightProcessInstance> getLightProcessInstances(ProcessDefinitionUUID processUUID, String queryList) throws RemoteException;

  Set<LightProcessInstance> getLightWeightProcessInstances(Set<ProcessDefinitionUUID> processUUIDs, String queryList) throws RemoteException;
  
  List<LightProcessInstance> getLightParentProcessInstances(int fromIndex, int pageSize, String queryList) throws RemoteException;
  
  Set<ProcessInstance> getProcessInstancesWithInstanceStates(Collection<InstanceState> instanceStates, String queryList) throws RemoteException;
  
  Set<ProcessInstance> getProcessInstancesWithTaskState(Collection<ActivityState> activityStates, String queryList) throws RemoteException;

  Set<ProcessInstance> getProcessInstances(ProcessDefinitionUUID processUUID, 
      String queryList) throws ProcessNotFoundException, RemoteException;

  int getNumberOfProcessInstances(String queryList) throws RemoteException;
  int getNumberOfParentProcessInstances(String queryList) throws RemoteException;
  
  ActivityInstance getActivityInstance(ActivityInstanceUUID
      activityUUID, String queryList) throws ActivityNotFoundException, RemoteException;

  LightActivityInstance getLightActivityInstance(ActivityInstanceUUID activityInstanceUUID, String queryList) throws ActivityNotFoundException, RemoteException;

  Set<ActivityInstance> getActivityInstances(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException;
  Set<LightActivityInstance> getLightActivityInstances(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException;
  
  List<LightTaskInstance> getLightTaskInstancesFromRoot(ProcessInstanceUUID rootInstanceUUID, String queryList) throws RemoteException;
  Map<ProcessInstanceUUID, List<LightTaskInstance>> getLightTaskInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, String queryList) throws RemoteException;
  
  List<LightActivityInstance> getLightActivityInstancesFromRoot(ProcessInstanceUUID rootInstanceUUID, String queryList) throws RemoteException;
  Map<ProcessInstanceUUID, List<LightActivityInstance>> getLightActivityInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, String queryList) throws RemoteException;
  Map<ProcessInstanceUUID,List<LightActivityInstance>> getLightActivityInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, ActivityState state, String queryList) throws RemoteException;
  Map<ProcessInstanceUUID, LightActivityInstance> getLightLastUpdatedActivityInstanceFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, boolean considerSystemTaks, String queryList) throws RemoteException;
  
  Set<ActivityInstance> getActivityInstances(ProcessInstanceUUID instanceUUID, String activityId, String queryList)
  throws InstanceNotFoundException, ActivityNotFoundException, RemoteException;
  Set<LightActivityInstance> getLightActivityInstances(ProcessInstanceUUID instanceUUID, String activityName, String queryList)
  throws InstanceNotFoundException, ActivityNotFoundException, RemoteException;
  
  TaskInstance getTask(ActivityInstanceUUID taskUUID, String queryList) throws TaskNotFoundException, RemoteException;
  
  boolean canExecuteTask(ActivityInstanceUUID taskUUID, String queryList) throws TaskNotFoundException, RemoteException;

  Set<TaskInstance> getTasks(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException;
  Set<LightTaskInstance> getLightTasks(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException;
  
  Collection<TaskInstance> getTaskList(ProcessInstanceUUID instanceUUID, ActivityState taskState, String queryList) throws InstanceNotFoundException, RemoteException;
  Collection<LightTaskInstance> getLightTaskList(ProcessInstanceUUID instanceUUID, ActivityState taskState, String queryList) throws InstanceNotFoundException, RemoteException;
  
  Collection<TaskInstance> getTaskList(ProcessInstanceUUID instanceUUID, Collection<ActivityState> taskStates, String queryList) throws InstanceNotFoundException, RemoteException;
  Collection<LightTaskInstance> getLightTaskList(ProcessInstanceUUID instanceUUID, Collection<ActivityState> taskStates, String queryList) throws InstanceNotFoundException, RemoteException;
  
  Collection<TaskInstance> getTaskList(ActivityState taskState, String queryList) throws RemoteException;
  Collection<LightTaskInstance> getLightTaskList(ActivityState taskState, String queryList) throws RemoteException;

  ActivityInstanceUUID getOneTask(ProcessInstanceUUID instanceUUID, ActivityState taskState, String queryList) throws RemoteException;
  
  ActivityInstanceUUID getOneTask(ProcessDefinitionUUID processUUID, ActivityState taskState, String queryList) throws RemoteException;
  
  ActivityInstanceUUID getOneTask(ActivityState taskState, String queryList) throws RemoteException;
  
  Object getActivityInstanceVariable(ActivityInstanceUUID activityUUID,
      String variableId, String queryList) throws ActivityNotFoundException, VariableNotFoundException, RemoteException;

  ActivityState getActivityInstanceState(ActivityInstanceUUID activityUUID,
      String queryList) throws ActivityNotFoundException, RemoteException;
  
  Map<String, Object> getActivityInstanceVariables(ActivityInstanceUUID
      activityUUID, String queryList)throws ActivityNotFoundException, RemoteException;

  Object getProcessInstanceVariable(ProcessInstanceUUID instanceUUID,
      String variableId, String queryList) throws InstanceNotFoundException, VariableNotFoundException, RemoteException;

  Map<String, Object> getProcessInstanceVariables(ProcessInstanceUUID
      instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException;

  Object getVariable(ActivityInstanceUUID activityUUID,
      String variableId, String queryList) throws ActivityNotFoundException, VariableNotFoundException, RemoteException;
  Map<String, Object> getVariables(ActivityInstanceUUID activityUUID, String queryList)
    throws InstanceNotFoundException, ActivityNotFoundException, RemoteException;
  
  Collection<TaskInstance> getTaskList(ProcessInstanceUUID instanceUUID, String userId, ActivityState taskState, String queryList) throws InstanceNotFoundException, RemoteException;
  Collection<LightTaskInstance> getLightTaskList(ProcessInstanceUUID instanceUUID, String userId, ActivityState taskState, String queryList) throws InstanceNotFoundException, RemoteException;
  
  Collection<TaskInstance> getTaskList(String userId, ActivityState taskState, String queryList) throws RemoteException;
  Collection<LightTaskInstance> getLightTaskList(String userId, ActivityState taskState, String queryList) throws RemoteException;

  /* COMMENT FEED */
  List<Comment> getCommentFeed(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException;
  List<Comment> getActivityInstanceCommentFeed(ActivityInstanceUUID activityUUID, String queryList) throws InstanceNotFoundException, RemoteException;
  List<Comment> getProcessInstanceCommentFeed(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException;

  int getNumberOfActivityInstanceComments(ActivityInstanceUUID activityUUID, String queryList) throws InstanceNotFoundException, RemoteException;
  Map<ActivityInstanceUUID, Integer> getNumberOfActivityInstanceComments(Set<ActivityInstanceUUID> activityUUIDs, String queryList) throws InstanceNotFoundException, RemoteException;
  int getNumberOfProcessInstanceComments(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException;
  int getNumberOfComments(ProcessInstanceUUID instanceUUID, String queryList) throws InstanceNotFoundException, RemoteException;

  Set<ProcessInstance> getUserInstances(String queryList) throws RemoteException;
  Set<LightProcessInstance> getLightUserInstances(String queryList) throws RemoteException;

  Set<String> getAttachmentNames(ProcessInstanceUUID instanceUUID, String queryList) throws RemoteException;
  Collection<AttachmentInstance> getLastAttachments(ProcessInstanceUUID instanceUUID, Set<String> attachmentNames, String queryList) throws RemoteException;
  Collection<AttachmentInstance> getLastAttachments(ProcessInstanceUUID instanceUUID, String regex, String queryList) throws RemoteException;
  List<AttachmentInstance> getAttachments(ProcessInstanceUUID instanceUUID, String attachmentName, String queryList) throws RemoteException;
  byte[] getAttachmentValue(AttachmentInstance attachmentInstance, String queryList) throws RemoteException;
  AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName, String queryList) throws RemoteException;
  AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName, Date date, String queryList) throws RemoteException;
  
  AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName, ActivityInstanceUUID activityUUID, String queryList)
  throws ActivityNotFoundException, RemoteException;
  
  LightTaskInstance getLightTaskInstance(ActivityInstanceUUID activityUUID, String queryList)
  throws TaskNotFoundException, RemoteException;
  
  Set<String> getTaskCandidates(final ActivityInstanceUUID taskUUID, String queryList) throws RemoteException, TaskNotFoundException;
  Map<ActivityInstanceUUID, Set<String>> getTaskCandidates(final Set<ActivityInstanceUUID> taskUUIDs, String queryList) throws RemoteException, TaskNotFoundException;
  
//  Set<? extends AbstractUUID> getItems(String entityID, RuleType type);
//  Set<ProcessDefinitionUUID> getProcesses(String entityID, RuleType type);
}
