/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.privilege.impl;

import java.util.HashSet;
import java.util.Set;

import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.util.Misc;

/**
 * @author Nicolas Chabanoles
 * 
 */
public abstract class ProcessRuleImpl extends RuleImpl {

  private static final long serialVersionUID = -1096929781625084230L;

  public ProcessRuleImpl(RuleType ruleType) {
    super(ruleType);
  }
  
  public ProcessRuleImpl(String name, String label, String description, Set<ProcessDefinitionUUID> items, RuleType ruleType) {
    super(name, label, description, items, ruleType);
  }

  public ProcessRuleImpl(ProcessRuleImpl src) {
    super(src);
  }
  
  public void addProcesses(Set<ProcessDefinitionUUID> items) {
    Misc.checkArgsNotNull(items);
    Set<String> processes = new HashSet<String>();
    for (ProcessDefinitionUUID processDefinitionUUID : items) {
      processes.add(processDefinitionUUID.getValue());
    }
    addItems(processes);

  }
  
  public void removeProcesses(Set<ProcessDefinitionUUID> items) {
    Misc.checkArgsNotNull(items);
    Set<String> processes = new HashSet<String>();
    for (ProcessDefinitionUUID processDefinitionUUID : items) {
      processes.add(processDefinitionUUID.getValue());
    }
    removeExceptions(processes);

  }

  public void setProcesses(Set<ProcessDefinitionUUID> processes) {
    // clear the existing processes.
    removeExceptions(getItems());
    addProcesses(processes);
  }
  
  public Set<ProcessDefinitionUUID> getProcesses() {
    
    Set<ProcessDefinitionUUID> processUUIDs = new HashSet<ProcessDefinitionUUID>();
    for (String definitionUUID : getItems()) {
      processUUIDs.add(new ProcessDefinitionUUID(definitionUUID));
    }
    return processUUIDs;
  }
}
