/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.runtime.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.ow2.bonita.env.Environment;
import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition.ProcessState;
import org.ow2.bonita.facade.privilege.PrivilegePolicy;
import org.ow2.bonita.facade.privilege.Rule;
import org.ow2.bonita.facade.privilege.Rule.RuleType;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.light.LightProcessDefinition;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.Command;

public class WebGetStartableProcessesCommand implements
    Command<Set<LightProcessDefinition>> {

  private static final long serialVersionUID = 1L;
  private ArrayList<String> entities;

  public WebGetStartableProcessesCommand(String... entities) {
    super();
    this.entities = new ArrayList<String>(Arrays.asList(entities));
  }

  public Set<LightProcessDefinition> execute(Environment environment)
      throws Exception {
    final ManagementAPI managementAPI = AccessorUtil.getManagementAPI();
    final QueryDefinitionAPI queryDefinitionAPI = AccessorUtil.getQueryDefinitionAPI(AccessorUtil.QUERYLIST_JOURNAL_KEY);

    Set<Rule> applicableRules = new HashSet<Rule>();
    Set<String> exceptions = new HashSet<String>();
    for (String entityID : this.entities) {
      applicableRules = managementAPI.getApplicableRules(entityID,
          RuleType.PROCESS_START);
      for (Rule rule : applicableRules) {
        exceptions.addAll(rule.getItems());
      }
    }

    Set<ProcessDefinitionUUID> processUUIDs = new HashSet<ProcessDefinitionUUID>();
    for (String processUUID : exceptions) {
      processUUIDs.add(new ProcessDefinitionUUID(processUUID));
    }
    PrivilegePolicy currentPolicy = managementAPI.getDefaultPolicy();
    switch (currentPolicy) {
      case ALLOW_BY_DEFAULT:
        // The exceptions are the processes the entity cannot start.
        if(processUUIDs!=null && !processUUIDs.isEmpty()){
          List<LightProcessDefinition> tempResult = queryDefinitionAPI.getAllLightProcessesExcept(processUUIDs, 0, queryDefinitionAPI.getNumberOfProcesses());
          Set<LightProcessDefinition> result = new HashSet<LightProcessDefinition>();
          for (LightProcessDefinition lightProcessDefinition : tempResult) {
            if (lightProcessDefinition.getState() == ProcessState.ENABLED) {
              result.add(lightProcessDefinition);
            }
          }
          return result;
        } else {
          return queryDefinitionAPI.getLightProcesses(ProcessState.ENABLED);
        }
        
      case DENY_BY_DEFAULT:
        // The exceptions are the processes the entity can start.
        if (processUUIDs.size() > 0) {
          Set<LightProcessDefinition> tempResult2 = queryDefinitionAPI
              .getLightProcesses(processUUIDs);
          if (tempResult2 == null || tempResult2.isEmpty()) {
            return Collections.emptySet();
          } else {
            Set<LightProcessDefinition> result2 = new HashSet<LightProcessDefinition>();
            for (LightProcessDefinition lightProcessDefinition : tempResult2) {
              if (lightProcessDefinition.getState() == ProcessState.ENABLED) {
                result2.add(lightProcessDefinition);
              }
            }
            return result2;
          }
        } else {
          return Collections.emptySet();
        }
      default:
        throw new IllegalArgumentException();
    }
    // Set<ProcessDefinitionUUID> processUUIDs = new
    // HashSet<ProcessDefinitionUUID>();
    // for (String entityID : this.entities) {
    // processUUIDs.addAll(FacadeUtil.getAllowedProcessUUIDsFor(entityID,
    // RuleType.PROCESS_START));
    // }
    // final QueryDefinitionAPI queryDefinitionAPI = AccessorUtil
    // .getQueryDefinitionAPI();
    // Set<LightProcessDefinition> tempResult = queryDefinitionAPI
    // .getLightProcesses(processUUIDs);
    //
    // if (tempResult == null || tempResult.isEmpty()) {
    // return Collections.emptySet();
    // }
    //
    // // Do not return processes that are not enabled.
    // Set<LightProcessDefinition> result = new
    // HashSet<LightProcessDefinition>();
    // for (LightProcessDefinition lightProcessDefinition : tempResult) {
    // if (lightProcessDefinition.getState() == ProcessState.ENABLED) {
    // result.add(lightProcessDefinition);
    // }
    // }
    // return result;

  }
}
