/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.runtime.impl;

import java.io.Serializable;

import org.ow2.bonita.facade.runtime.Label;

/**
 * This class defines the concept of 'Label'. A Label is a human readable text
 * value that can be associated to a 'Case'. This joins the concept of 'key
 * words' that can be associated to documents to help the indexing.
 * 
 * @author Nicolas Chabanoles
 * 
 */
public class LabelImpl implements Serializable, Comparable<LabelImpl>, Label {

  private static final long serialVersionUID = 888850443702378668L;

  protected long dbid;

  protected String labelName;
  protected String ownerName;
  protected String editableCSSStyleName;
  protected String readonlyCSSStyleName;
  protected String previewCSSStyleName;
  protected boolean isVisible;
  protected boolean hasToBeDisplayed;
  protected String iconCSSStyle;
  protected int displayOrder;
  protected boolean isSystemLabel;

  protected LabelImpl() {
    super();
    // Mandatory for serialization.
  }

  public LabelImpl(String labelName, String ownerName,
      String editableCSSStyleName, String readonlyCSSStyleName,
      String previewCSSStyleName, boolean isVisible, boolean hasToBeDisplayed,
      String iconCSSStyle, int displayOrder,
      boolean isSystemLabel) {
    super();
    this.labelName = labelName;
    this.ownerName = ownerName;
    this.editableCSSStyleName = editableCSSStyleName;
    this.readonlyCSSStyleName = readonlyCSSStyleName;
    this.previewCSSStyleName = previewCSSStyleName;
    this.isVisible = isVisible;
    this.hasToBeDisplayed = hasToBeDisplayed;
    this.iconCSSStyle = iconCSSStyle;
    this.displayOrder = displayOrder;
    this.isSystemLabel = isSystemLabel;
  }

  public LabelImpl(final Label src) {
    super();
    this.labelName = src.getLabelName();
    this.ownerName = src.getOwnerName();
    this.editableCSSStyleName = src.getEditableCSSStyleName();
    this.readonlyCSSStyleName = src.getReadonlyCSSStyleName();
    this.previewCSSStyleName = src.getPreviewCSSStyleName();
    this.isVisible = src.isVisible();
    this.hasToBeDisplayed = src.isHasToBeDisplayed();
    this.iconCSSStyle = src.getIconCSSStyle();
    this.displayOrder = src.getDisplayOrder();
    this.isSystemLabel = src.isSystemLabel();

    //to prevent join tables, this set is not copied
    /*
    if (!src.getCases().isEmpty()) {
      this.cases = new HashSet<Case>();
      for (Case case_ : ((LabelImpl)src).getInternalCases()) {
        this.cases.add(new CaseImpl(case_));
      }
    }
    */
  }

  @Override
  public String toString() {
    final StringBuffer sb = new StringBuffer();
    sb.append(this.getClass().getSimpleName());
    sb.append(", name= " + getLabelName());
    sb.append(", owner= " + getOwnerName());
    return sb.toString();
  }
  public String getLabelName() {
    return this.labelName;
  }

  public void setLabelName(String aLabelName) {
    this.labelName = aLabelName;
  }

  public String getOwnerName() {
    return this.ownerName;
  }

  public void setOwnerName(String aOwnerName) {
    this.ownerName = aOwnerName;
  }

  public String getEditableCSSStyleName() {
    return this.editableCSSStyleName;
  }

  public void setEditableCSSStyleName(String aEditableCSSStyleName) {
    this.editableCSSStyleName = aEditableCSSStyleName;
  }

  public String getReadonlyCSSStyleName() {
    return this.readonlyCSSStyleName;
  }

  public void setReadonlyCSSStyleName(String aReadonlyCSSStyleName) {
    this.readonlyCSSStyleName = aReadonlyCSSStyleName;
  }

  public String getPreviewCSSStyleName() {
    return this.previewCSSStyleName;
  }

  public void setPreviewCSSStyleName(String aPreviewCSSStyleName) {
    this.previewCSSStyleName = aPreviewCSSStyleName;
  }

  public boolean isVisible() {
    return this.isVisible;
  }

  public void setVisible(boolean aIsVisible) {
    this.isVisible = aIsVisible;
  }

  public boolean isHasToBeDisplayed() {
    return this.hasToBeDisplayed;
  }

  public void setHasToBeDisplayed(boolean aHasToBeDisplayed) {
    this.hasToBeDisplayed = aHasToBeDisplayed;
  }

  public String getIconCSSStyle() {
    return this.iconCSSStyle;
  }

  public void setIconCSSStyle(String aIconCSSStyle) {
    this.iconCSSStyle = aIconCSSStyle;
  }

  public int getDisplayOrder() {
    return this.displayOrder;
  }

  public void setDisplayOrder(int aDisplayOrder) {
    this.displayOrder = aDisplayOrder;
  }

  public boolean isSystemLabel() {
    return this.isSystemLabel;
  }

  public void setSystemLabel(boolean aIsSystemLabel) {
    this.isSystemLabel = aIsSystemLabel;
  }

  @Override
  public boolean equals(Object anObject) {
    if (anObject == null) {
      return false;
    }
    if (!(anObject instanceof LabelImpl)) {
      return false;
    }
    LabelImpl other = (LabelImpl) anObject;
    return getLabelName().equals(other.getLabelName()) && getOwnerName().equals(other.getOwnerName());
  }

  @Override
  public int hashCode() {
    return labelName.hashCode();
  }
  public int compareTo(LabelImpl o) {
    int thisVal = this.displayOrder;
    int anotherVal = o.displayOrder;
    return (thisVal < anotherVal ? -1 : (thisVal == anotherVal ? 0 : 1));
  }
}
