/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.parsing.def.binding;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.bonita.building.XmlDef;
import org.ow2.bonita.util.ProcessBuilder;
import org.ow2.bonita.util.xml.Parse;
import org.ow2.bonita.util.xml.Parser;
import org.ow2.bonita.util.xml.XmlUtil;
import org.w3c.dom.Element;

/**
 * @author Anthony Birembaut
 */
public class ProcessBinding extends ElementBinding {
  
  public ProcessBinding() {
    super(XmlDef.PROCESS);
  }

  private static final Logger LOGGER = Logger.getLogger(ProcessBinding.class.getName());

  public Object parse(final Element processElement, final Parse parse, final Parser parser) {
    if (LOGGER.isLoggable(Level.FINE)) {
      LOGGER.fine("parsing element = " + processElement);
    }
    
    String name = null;
    
    try {
      name = XmlUtil.attribute(processElement, XmlDef.NAME);
      final String version = XmlUtil.attribute(processElement, XmlDef.VERSION);
      final String description = getChildTextContent(processElement, XmlDef.DESCRIPTION);
      final String label = getChildTextContent(processElement, XmlDef.LABEL);
  
      ProcessBuilder processBuilder = ProcessBuilder.createProcess(name, version);
      processBuilder.addDescription(description);
      processBuilder.addLabel(label);
  
      parse.pushObject(processBuilder);
      
      parseElementList(processElement, XmlDef.PARTICIPANTS, XmlDef.PARTICIPANT, parse, parser);
      parseElementList(processElement, XmlDef.DATA_FIELDS, XmlDef.DATA_FIELD, parse, parser);
      parseElementList(processElement, XmlDef.ATTACHMENTS, XmlDef.ATTACHMENT, parse, parser);
      parseElementList(processElement, XmlDef.CONNECTORS, XmlDef.CONNECTOR, parse, parser);
      parseElementList(processElement, XmlDef.ACTIVITIES, XmlDef.ACTIVITY, parse, parser);
      parseElementList(processElement, XmlDef.TRANSITIONS, XmlDef.TRANSITION, parse, parser);
      
      parse.popObject();
  
      return processBuilder.done();
    } catch (Exception e) {
      parse.addProblem("Error parsing Process " + name, e);
      return null;
    }
  }
}
